@@ -19,7 +19,7 @@
 #' Export files
 #'
 #' This function provides a general interface to export
-#' \code{\link[MALDIquant]{AbstractMassObject-class}} objects (e.g. 
+#' \code{\link[MALDIquant]{AbstractMassObject-class}} objects (e.g.
 #' \code{\link[MALDIquant]{MassSpectrum-class}},
 #' \code{\link[MALDIquant]{MassPeaks-class}})
 #' into different file formats.
@@ -30,14 +30,18 @@
 #'  tab \tab \code{\link[MALDIquantForeign]{exportTab}} \cr
 #'  csv \tab \code{\link[MALDIquantForeign]{exportCsv}} \cr
 #'  msd \tab \code{\link[MALDIquantForeign]{exportMsd}} \cr
+#'  mzML \tab \code{\link[MALDIquantForeign]{exportMzMl}} \cr
 #' }
 #'
-#' @param x a \code{\link[MALDIquant]{AbstractMassObject-class}} object or a 
+#' @usage
+#' \S4method{export}{AbstractMassObject}(x, file, type="auto", force=FALSE, \ldots)
+#'
+#' @param x a \code{\link[MALDIquant]{AbstractMassObject-class}} object or a
 #'  \code{list} of \code{\link[MALDIquant]{AbstractMassObject-class}} objects.
 #' @param file \code{character}, file name.
 #' @param path \code{character}, path to directory in which the \code{list} of
 #'  \code{\link[MALDIquant]{AbstractMassObject-class}} would be exported.
-#' @param type \code{character}, file format. If \code{type} is set to 
+#' @param type \code{character}, file format. If \code{type} is set to
 #'  \dQuote{auto} the file extension is used.
 #' @param force \code{logical}, If \code{TRUE} the \code{file} would be
 #'  overwritten or \code{path} would be created.
@@ -45,7 +49,7 @@
 #'
 #' @seealso
 #' \code{\link[MALDIquant]{MassPeaks-class}},
-#' \code{\link[MALDIquant]{MassSpectrum-class}} 
+#' \code{\link[MALDIquant]{MassSpectrum-class}}
 #' @author Sebastian Gibb
 #' @references \url{http://strimmerlab.org/software/maldiquant/}
 #' @examples
@@ -58,40 +62,26 @@
 #'           createMassSpectrum(mass=1:5, intensity=1:5))
 #'
 #' ## export a single spectrum
-#' export(s[[1]], file="spectrum.csv") 
+#' export(s[[1]], file="spectrum.csv")
 #' ## identical to exportCsv(s[[1]], file="spectrum.csv")
 #'
 #' ## export a list of spectra
-#' export(s, path="spectra", type="csv") 
+#' export(s, path="spectra", type="csv")
 #' ## identical to exportCsv(s, path="spectra")
 #' }
 #'
-#' @aliases export,AbstractMassObject-method export,list-method
+#' @aliases export export,AbstractMassObject-method export,list-method
 #' @rdname export-methods
+#' @docType methods
 #' @export
 setMethod(f="export",
   signature=signature(x="AbstractMassObject"),
   definition=function(x, file, type="auto", force=FALSE, ...) {
-
-  if (file.exists(file) && !force) {
-    stop("File already exists! Use ", sQuote("force=TRUE"), " to overwrite it.")
-  }
-
-  if (missing(type) || pmatch(tolower(type), "auto", nomatch=0,
-                              duplicates.ok=FALSE)) {
-    type <- .fileExtension(file)
-  }
-
-  i <- pmatch(tolower(type), exportFormats$type, nomatch=0, duplicates.ok=FALSE)
-
-  if (i <= 0) {
-    stop("File type ", sQuote(type), " is not supported!")
-  } else {
-    handler <- exportFormats$handler[i]
-    return(do.call(handler, list(x=x, file=file, ...)))
-  }
+  return(.exportToFile(x=x, file=file, type=type, force=force, ...))
 })
 
+#' @usage
+#' \S4method{export}{list}(x, path, type, force=FALSE, \ldots)
 #' @rdname export-methods
 #' @export
 setMethod(f="export",
@@ -100,56 +90,21 @@ setMethod(f="export",
 
   stopifnot(isMassObjectList(x))
 
-  if (!file.exists(path) && force) {
-    dir.create(path, showWarnings=FALSE, recursive=TRUE)  
-  }
+  onefileSupport <- exportFormats$type[exportFormats$onefile]
+  isFile <- (missing(type) &&
+             tolower(.fileExtension(path)) %in% onefileSupport)
 
-  if (!file.exists(path)) {
-    stop("Directory ", sQuote(path), " doesn't exist!")
-  }
-
-  if (!file.info(path)$isdir) {
-    stop(sQuote(path), " is no directory!")
-  }
-
-  ## stop if directory isn't writeable
-  if (file.access(path, 2) != 0) {
-    stop("No permissions to write into ", sQuote(path), "!")
-  }
-
-  i <- pmatch(tolower(type), exportFormats$type, nomatch=0, duplicates.ok=FALSE)
-
-  if (i <= 0) {
-    stop("File type ", sQuote(type), " is not supported!")
+  if (isFile) {
+    return(.exportToFile(x=x, file=path, type=type, force=force, ...))
   } else {
-    filenames <- .composeFilename(x, fileExtension=exportFormats$extension[i])
-    filenames <- file.path(path, filenames)
-     
-    optArgs <- list(...)
-    peaks <- list()
-
-    if (hasArg(peaks)) {
-      peaks <- optArgs$peaks 
-      optArgs$peaks <- NULL
-    }
-
-    for (i in seq(along=x)) {
-      arguments <- list(x=x[[i]], file=filenames[i], type=type, force=force)
-      arguments <- modifyList(arguments, optArgs)
-
-      if (length(peaks)) {
-        arguments$peaks <- peaks[[i]]
-      }
-      do.call(export, arguments)
-    }
+    return(.exportToDir(x=x, path=path, type=type, force=force, ...))
   }
-  invisible()
 })
 
 #' Export to text files
 #'
-#' This function exports 
-#' \code{\link[MALDIquant]{AbstractMassObject-class}} objects (e.g. 
+#' This function exports
+#' \code{\link[MALDIquant]{AbstractMassObject-class}} objects (e.g.
 #' \code{\link[MALDIquant]{MassSpectrum-class}},
 #' \code{\link[MALDIquant]{MassPeaks-class}})
 #' into different text file formats.
@@ -159,7 +114,10 @@ setMethod(f="export",
 #' with different defaults (\code{sep="\t"} in \code{exportTab} and
 #' \code{sep=","} in \code{exportCsv}).
 #'
-#' @param x a \code{\link[MALDIquant]{AbstractMassObject-class}} object or a 
+#' @usage
+#' \S4method{exportTab}{AbstractMassObject}(x, file, force=FALSE, \ldots)
+#'
+#' @param x a \code{\link[MALDIquant]{AbstractMassObject-class}} object or a
 #'  \code{list} of \code{\link[MALDIquant]{AbstractMassObject-class}} objects.
 #' @param file \code{character}, file name.
 #' @param path \code{character}, path to directory in which the \code{list} of
@@ -184,15 +142,16 @@ setMethod(f="export",
 #'           createMassSpectrum(mass=1:5, intensity=1:5))
 #'
 #' ## export a single spectrum
-#' exportTab(s[[1]], file="spectrum.tab") 
+#' exportTab(s[[1]], file="spectrum.tab")
 #'
 #' ## export a list of spectra and use ; as separator
-#' exportCsv(s, path="spectra", sep=";", force=TRUE) 
+#' exportCsv(s, path="spectra", sep=";", force=TRUE)
 #' }
 #'
-#' @aliases exportTab,AbstractMassObject-method exportTab,list-method
-#' exportCsv,AbstractMassObject-method exportCsv,list-method
+#' @aliases exportTab exportTab,AbstractMassObject-method exportTab,list-method
+#' exportCsv exportCsv,AbstractMassObject-method exportCsv,list-method
 #' @rdname exportTab-methods
+#' @docType methods
 #' @export
 setMethod(f="exportTab",
           signature=signature(x="AbstractMassObject"),
@@ -200,6 +159,8 @@ setMethod(f="exportTab",
   export(x, file=file, type="tab", force=force, ...)
 })
 
+#' @usage
+#' \S4method{exportTab}{list}(x, path, force=FALSE, \ldots)
 #' @rdname exportTab-methods
 #' @export
 setMethod(f="exportTab",
@@ -208,6 +169,8 @@ setMethod(f="exportTab",
   export(x, path=path, type="tab", force=force, ...)
 })
 
+#' @usage
+#' \S4method{exportCsv}{AbstractMassObject}(x, file, force=FALSE, \ldots)
 #' @rdname exportTab-methods
 #' @export
 setMethod(f="exportCsv",
@@ -216,6 +179,8 @@ setMethod(f="exportCsv",
   export(x, file=file, type="csv", force=force, ...)
 })
 
+#' @usage
+#' \S4method{exportCsv}{list}(x, path, force=FALSE, \ldots)
 #' @rdname exportTab-methods
 #' @export
 setMethod(f="exportCsv",
@@ -226,18 +191,21 @@ setMethod(f="exportCsv",
 
 #' Export to MSD files
 #'
-#' This function exports 
-#' \code{\link[MALDIquant]{AbstractMassObject-class}} objects (e.g. 
+#' This function exports
+#' \code{\link[MALDIquant]{AbstractMassObject-class}} objects (e.g.
 #' \code{\link[MALDIquant]{MassSpectrum-class}},
 #' \code{\link[MALDIquant]{MassPeaks-class}})
 #' into mMass MSD files.
 #'
-#' @param x a \code{\link[MALDIquant]{MassSpectrum-class}} object or a 
+#' @usage
+#' \S4method{exportMsd}{MassSpectrum}(x, file, force=FALSE, peaks, \ldots)
+#'
+#' @param x a \code{\link[MALDIquant]{MassSpectrum-class}} object or a
 #'  \code{list} of \code{\link[MALDIquant]{MassSpectrum-class}} objects.
 #' @param file \code{character}, file name.
 #' @param path \code{character}, path to directory in which the \code{list} of
 #'  \code{\link[MALDIquant]{AbstractMassObject-class}} would be exported.
-#' @param peaks a \code{\link[MALDIquant]{MassPeaks-class}} object or a 
+#' @param peaks a \code{\link[MALDIquant]{MassPeaks-class}} object or a
 #'  \code{list} of \code{\link[MALDIquant]{MassPeaks-class}} objects.
 #' @param force \code{logical}, If \code{TRUE} the \code{file} would be
 #'  overwritten or \code{path} would be created.
@@ -249,7 +217,7 @@ setMethod(f="exportCsv",
 #'
 #' @author Sebastian Gibb
 #' @references \url{http://strimmerlab.org/software/maldiquant/}, \cr
-#' mMass homepage \url{http://mmass.org/}
+#' mMass homepage: \url{http://mmass.org/}
 #' @examples
 #'
 #' \dontrun{
@@ -262,23 +230,22 @@ setMethod(f="exportCsv",
 #'           createMassPeaks(mass=4:5, intensity=4:5, snr=1:2))
 #'
 #' ## export a single spectrum
-#' exportMsd(s[[1]], file="spectrum.msd") 
+#' exportMsd(s[[1]], file="spectrum.msd")
 #'
 #' ## export a single spectrum with corresponding peaks
-#' exportMsd(s[[1]], file="spectrum.msd", peaks=p[[1]]) 
+#' exportMsd(s[[1]], file="spectrum.msd", peaks=p[[1]])
 #'
 #' ## export a list of spectra with corresponding peaks
-#' exportMsd(s, path="spectra", peaks=p, force=TRUE) 
+#' exportMsd(s, path="spectra", peaks=p, force=TRUE)
 #' }
 #'
-#' @aliases exportMsd,MassSpectrum-method exportMsd,list-method
+#' @aliases exportMsd exportMsd,MassSpectrum-method exportMsd,list-method
 #' @rdname exportMsd-methods
+#' @docType methods
 #' @export
 setMethod(f="exportMsd",
           signature=signature(x="MassSpectrum"),
           definition=function(x, file, force=FALSE, peaks, ...) {
-  stopifnot(isMassSpectrum(x))
-
   if (!missing(peaks)) {
     stopifnot(isMassPeaks(peaks))
     export(x, file=file, type="msd", force=force, peaks=peaks,  ...)
@@ -287,6 +254,8 @@ setMethod(f="exportMsd",
   }
 })
 
+#' @usage
+#' \S4method{exportMsd}{list}(x, path, force=FALSE, peaks, \ldots)
 #' @rdname exportMsd-methods
 #' @export
 setMethod(f="exportMsd",
@@ -302,3 +271,67 @@ setMethod(f="exportMsd",
   }
 })
 
+#' Export to mzML files
+#'
+#' This function exports
+#' \code{\link[MALDIquant]{AbstractMassObject-class}} objects (e.g.
+#' \code{\link[MALDIquant]{MassSpectrum-class}},
+#' \code{\link[MALDIquant]{MassPeaks-class}})
+#' into mzML files.
+#'
+#' @usage
+#' \S4method{exportMzMl}{MassSpectrum}(x, file, force=FALSE, \ldots)
+#'
+#' @param x a \code{\link[MALDIquant]{MassSpectrum-class}} object or a
+#'  \code{list} of \code{\link[MALDIquant]{MassSpectrum-class}} objects.
+#' @param file \code{character}, file name.
+#' @param path \code{character}, path to directory in which the \code{list} of
+#'  \code{\link[MALDIquant]{AbstractMassObject-class}} would be exported.
+#' @param force \code{logical}, If \code{TRUE} the \code{file} would be
+#'  overwritten or \code{path} would be created.
+#' @param \ldots arguments to be passed to \code{\link[utils]{write.table}}.
+#'
+#' @seealso
+#' \code{\link[MALDIquant]{MassPeaks-class}},
+#' \code{\link[MALDIquant]{MassSpectrum-class}}
+#'
+#' @author Sebastian Gibb
+#' @references \url{http://strimmerlab.org/software/maldiquant/}, \cr
+#' HUPO Proteomics Standards Inititative mzML 1.1.0 Specification:
+#' \url{http://www.psidev.info/mzml_1_0_0}
+#' @examples
+#'
+#' \dontrun{
+#' library("MALDIquant")
+#' library("MALDIquantForeign")
+#'
+#' s <- list(createMassSpectrum(mass=1:5, intensity=1:5),
+#'           createMassSpectrum(mass=1:5, intensity=1:5))
+#'
+#' ## export a single spectrum
+#' exportMzMl(s[[1]], file="spectrum.mzML")
+#'
+#' ## export a list of spectra with corresponding peaks
+#' exportMzMl(s, path="spectra.mzMl")
+#' }
+#'
+#' @aliases exportMzMl exportMzMl,MassSpectrum-method exportMzMl,list-method
+#' @rdname exportMzMl-methods
+#' @docType methods
+#' @export
+setMethod(f="exportMzMl",
+          signature=signature(x="MassSpectrum"),
+          definition=function(x, file, force=FALSE, ...) {
+  export(x, file=file, type="mzml", force=force, ...)
+})
+
+#' @usage
+#' \S4method{exportMzMl}{list}(x, path, force=FALSE, \ldots)
+#' @rdname exportMzMl-methods
+#' @export
+setMethod(f="exportMzMl",
+          signature=signature(x="list"),
+          definition=function(x, path, force=FALSE, ...) {
+  export(x, path=path, type="mzml", force=force, ...)
+})
+
