@@ -17,17 +17,23 @@
 ## along with MALDIquantForeign. If not, see <http://www.gnu.org/licenses/>
 
 #' @keywords internal
-.list.files <- function(path, pattern, recursive=TRUE, ignore.case=FALSE) {
+.list.files <- function(path, pattern, recursive=TRUE, ignore.case=TRUE) {
   return(normalizePath(list.files(path=path, pattern=pattern,
                                   recursive=recursive, ignore.case=ignore.case,
                                   full.names=TRUE)))
 }
 
 #' @keywords internal
-.files <- function(path, pattern, ...) {
+.files <- function(path, pattern, ignore.case=TRUE, ...) {
   isDir <- file.info(path)$isdir
 
   files <- normalizePath(path[!isDir])
-  files <- c(files, .list.files(path=path[isDir], pattern=pattern, ...))
-  return(files)
+
+  isMatching <- unlist(regexpr(pattern=pattern, text=basename(files),
+                               ignore.case=ignore.case)) != -1
+
+  files <- files[isMatching]
+  files <- c(files, .list.files(path=path[isDir], pattern=pattern,
+                                ignore.case=ignore.case, ...))
+  return(unique(files))
 }
